import { targets, getHeaderName } from './chunk-XPUGYZ4Y.mjs';
export { addTarget, addTargetClient } from './chunk-XPUGYZ4Y.mjs';
import { parse, format } from 'url';
import { map } from 'event-stream';
import FormData from 'form-data';
import { stringify } from 'qs';

// src/helpers/form-data.ts
var carriage = "\r\n";
var dashes = "-".repeat(2);
var NAME = Symbol.toStringTag;
var isBlob = (object) => typeof object === "object" && typeof object.arrayBuffer === "function" && typeof object.type === "string" && typeof object.stream === "function" && typeof object.constructor === "function" && /^(Blob|File)$/.test(object[NAME]);
var getFooter = (boundary) => `${dashes}${boundary}${dashes}${carriage.repeat(2)}`;
var getHeader = (boundary, name, field) => {
  let header = "";
  header += `${dashes}${boundary}${carriage}`;
  header += `Content-Disposition: form-data; name="${name}"`;
  if (isBlob(field)) {
    header += `; filename="${field.name}"${carriage}`;
    header += `Content-Type: ${field.type || "application/octet-stream"}`;
  }
  return `${header}${carriage.repeat(2)}`;
};
var formDataIterator = function* (form, boundary) {
  for (const [name, value] of form) {
    yield getHeader(boundary, name, value);
    if (isBlob(value)) {
      yield* value.stream();
    } else {
      yield value;
    }
    yield carriage;
  }
  yield getFooter(boundary);
};

// src/helpers/reducer.ts
var reducer = (accumulator, pair) => {
  const currentValue = accumulator[pair.name];
  if (currentValue === void 0) {
    accumulator[pair.name] = pair.value;
    return accumulator;
  }
  if (Array.isArray(currentValue)) {
    currentValue.push(pair.value);
    return accumulator;
  }
  accumulator[pair.name] = [currentValue, pair.value];
  return accumulator;
};

// src/helpers/utils.ts
var availableTargets = () => Object.keys(targets).map((targetId) => ({
  ...targets[targetId].info,
  clients: Object.keys(targets[targetId].clientsById).map(
    (clientId) => targets[targetId].clientsById[clientId].info
  )
}));
var extname = (targetId) => targets[targetId]?.info.extname || "";

// src/index.ts
var isHarEntry = (value) => typeof value === "object" && "log" in value && typeof value.log === "object" && "entries" in value.log && Array.isArray(value.log.entries);
var HTTPSnippet = class {
  constructor(input, opts = {}) {
    this.requests = [];
    this.prepare = (harRequest, options) => {
      const request = {
        ...harRequest,
        fullUrl: "",
        uriObj: {},
        queryObj: {},
        headersObj: {},
        cookiesObj: {},
        allHeaders: {}
      };
      if (request.queryString && request.queryString.length) {
        request.queryObj = request.queryString.reduce(reducer, {});
      }
      if (request.headers && request.headers.length) {
        const http2VersionRegex = /^HTTP\/2/;
        request.headersObj = request.headers.reduce((accumulator, { name, value }) => {
          const headerName = http2VersionRegex.exec(request.httpVersion) ? name.toLocaleLowerCase() : name;
          return {
            ...accumulator,
            [headerName]: value
          };
        }, {});
      }
      if (request.cookies && request.cookies.length) {
        request.cookiesObj = request.cookies.reduceRight(
          (accumulator, { name, value }) => ({
            ...accumulator,
            [name]: value
          }),
          {}
        );
      }
      const cookies = request.cookies?.map(({ name, value }) => {
        if (options.harIsAlreadyEncoded) {
          return `${name}=${value}`;
        }
        return `${encodeURIComponent(name)}=${encodeURIComponent(value)}`;
      });
      if (cookies?.length) {
        request.allHeaders.cookie = cookies.join("; ");
      }
      switch (request.postData.mimeType) {
        case "multipart/mixed":
        case "multipart/related":
        case "multipart/form-data":
        case "multipart/alternative":
          request.postData.text = "";
          request.postData.mimeType = "multipart/form-data";
          if (request.postData?.params) {
            const form = new FormData();
            const isNativeFormData = typeof form[Symbol.iterator] === "function";
            const boundary = "---011000010111000001101001";
            if (!isNativeFormData) {
              form._boundary = boundary;
            }
            request.postData?.params.forEach((param) => {
              const name = param.name;
              const value = param.value || "";
              const filename = param.fileName || null;
              if (isNativeFormData) {
                if (isBlob(value)) {
                  form.append(name, value, filename);
                } else {
                  form.append(name, value);
                }
              } else {
                form.append(name, value, {
                  // @ts-expect-error TODO
                  filename,
                  // @ts-expect-error TODO
                  contentType: param.contentType || null
                });
              }
            });
            if (isNativeFormData) {
              for (const data of formDataIterator(form, boundary)) {
                request.postData.text += data;
              }
            } else {
              form.pipe(
                // @ts-expect-error TODO
                map((data) => {
                  request.postData.text += data;
                })
              );
            }
            request.postData.boundary = boundary;
            const contentTypeHeader = getHeaderName(request.headersObj, "content-type") || "content-type";
            request.headersObj[contentTypeHeader] = `multipart/form-data; boundary=${boundary}`;
          }
          break;
        case "application/x-www-form-urlencoded":
          if (!request.postData.params) {
            request.postData.text = "";
          } else {
            request.postData.paramsObj = request.postData.params.reduce(reducer, {});
            request.postData.text = stringify(request.postData.paramsObj);
          }
          break;
        case "text/json":
        case "text/x-json":
        case "application/json":
        case "application/x-json":
          request.postData.mimeType = "application/json";
          if (request.postData.text) {
            try {
              request.postData.jsonObj = JSON.parse(request.postData.text);
            } catch (e) {
              request.postData.mimeType = "text/plain";
            }
          }
          break;
      }
      const allHeaders = {
        ...request.allHeaders,
        ...request.headersObj
      };
      const urlWithParsedQuery = parse(request.url, true, true);
      request.queryObj = {
        ...request.queryObj,
        ...urlWithParsedQuery.query
      };
      let search;
      if (options.harIsAlreadyEncoded) {
        search = stringify(request.queryObj, {
          encode: false,
          indices: false
        });
      } else {
        search = stringify(request.queryObj, {
          indices: false
        });
      }
      const uriObj = {
        ...urlWithParsedQuery,
        query: request.queryObj,
        search,
        path: search ? `${urlWithParsedQuery.pathname}?${search}` : urlWithParsedQuery.pathname
      };
      const url = format({
        ...urlWithParsedQuery,
        query: null,
        search: null
      });
      const fullUrl = format({
        ...urlWithParsedQuery,
        ...uriObj
      });
      return {
        ...request,
        allHeaders,
        fullUrl,
        url,
        uriObj
      };
    };
    this.convert = (targetId, clientId, options) => {
      if (!options && clientId) {
        options = clientId;
      }
      const target = targets[targetId];
      if (!target) {
        return false;
      }
      const { convert } = target.clientsById[clientId || target.info.default];
      const results = this.requests.map((request) => convert(request, options));
      return results.length === 1 ? results[0] : results;
    };
    let entries = [];
    const options = {
      harIsAlreadyEncoded: false,
      ...opts
    };
    this.requests = [];
    if (isHarEntry(input)) {
      entries = input.log.entries;
    } else {
      entries = [
        {
          request: input
        }
      ];
    }
    entries.forEach(({ request }) => {
      const req = {
        bodySize: 0,
        headersSize: 0,
        headers: [],
        cookies: [],
        httpVersion: "HTTP/1.1",
        queryString: [],
        postData: {
          mimeType: request.postData?.mimeType || "application/octet-stream"
        },
        ...request
      };
      if (req.postData && !req.postData.mimeType) {
        req.postData.mimeType = "application/octet-stream";
      }
      this.requests.push(this.prepare(req, options));
    });
  }
};
/**
 * @license https://raw.githubusercontent.com/node-fetch/node-fetch/master/LICENSE.md
 *
 * The MIT License (MIT)
 *
 * Copyright (c) 2016 - 2020 Node Fetch Team
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Extracted from https://github.com/node-fetch/node-fetch/blob/64c5c296a0250b852010746c76144cb9e14698d9/src/utils/form-data.js
 */

export { HTTPSnippet, availableTargets, extname };
//# sourceMappingURL=out.js.map
//# sourceMappingURL=index.mjs.map